@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,19 +11,22 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
+import json
 import os
 import sys
-import json
 from contextlib import contextmanager
 
 from hypothesis.internal.reflection import proxies
 
+if False:
+    from typing import Set, Dict, Tuple  # noqa
+
 """
 This module implements a custom coverage system that records conditions and
 then validates that every condition has been seen to be both True and False
@@ -37,7 +40,7 @@ When not running with a magic environment variable set, this module disables
 itself and has essentially no overhead.
 """
 
-pretty_file_name_cache = {}
+pretty_file_name_cache = {}  # type: Dict[str, str]
 
 
 def pretty_file_name(f):
@@ -47,29 +50,28 @@ def pretty_file_name(f):
         pass
 
     parts = f.split(os.path.sep)
-    parts = parts[parts.index('hypothesis'):]
+    parts = parts[-parts[::-1].index("hypothesis") :]
     result = os.path.sep.join(parts)
     pretty_file_name_cache[f] = result
     return result
 
 
-IN_COVERAGE_TESTS = os.getenv('HYPOTHESIS_INTERNAL_COVERAGE') == 'true'
+IN_COVERAGE_TESTS = os.getenv("HYPOTHESIS_INTERNAL_COVERAGE") == "true"
 
 
 if IN_COVERAGE_TESTS:
-    log = open('branch-check', 'w')
-    written = set()
+    # By this point, "branch-check" should have already been deleted by the
+    # tox config. We can't delete it here because of #1718.
+
+    written = set()  # type: Set[Tuple[str, bool]]
 
     def record_branch(name, value):
         key = (name, value)
         if key in written:
             return
         written.add(key)
-        log.write(
-            json.dumps({'name': name, 'value': value})
-        )
-        log.write('\n')
-        log.flush()
+        with open("branch-check", "a") as log:
+            log.write(json.dumps({"name": name, "value": value}) + "\n")
 
     description_stack = []
 
@@ -79,14 +81,14 @@ if IN_COVERAGE_TESTS:
         # function, one for our actual caller, so we want to go two extra
         # stack frames up.
         caller = sys._getframe(depth + 2)
-        local_description = '%s at %s:%d' % (
+        local_description = "%s at %s:%d" % (
             name,
             pretty_file_name(caller.f_code.co_filename),
             caller.f_lineno,
         )
         try:
             description_stack.append(local_description)
-            description = ' in '.join(reversed(description_stack)) + ' passed'
+            description = " in ".join(reversed(description_stack)) + " passed"
             yield
             record_branch(description, True)
         except BaseException:
@@ -106,20 +108,16 @@ if IN_COVERAGE_TESTS:
             # depth of 2 because of the proxy function calling us.
             with check_block(f.__name__, 2):
                 return f(*args, **kwargs)
+
         return accept
+
+
 else:
+
     def check_function(f):
         return f
 
-    @contextmanager
+    # Mypy bug: https://github.com/python/mypy/issues/4117
+    @contextmanager  # type: ignore
     def check(name):
         yield
-
-
-class suppress_tracing(object):
-    def __enter__(self):
-        self.__original_trace = sys.gettrace()
-        sys.settrace(None)
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        sys.settrace(self.__original_trace)
